/*

  This file is a part of GRALE, a library to facilitate the simulation
  and inversion of gravitational lenses.

  Copyright (C) 2008-2012 Jori Liesenborgs

  Contact: jori.liesenborgs@gmail.com
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
*/

/**
 * \file plummerbackprojecttree.h
 */

#ifndef GRALE_PLUMMERBACKPROJECTTREE_H

#define GRALE_PLUMMERBACKPROJECTTREE_H

#include "graleconfig.h"
#include "backprojecttreebase.h"

namespace grale
{

class GRALE_IMPORTEXPORT PlummerBackProjectTree : public BackProjectTreeBase
{
public:
	PlummerBackProjectTree();
	~PlummerBackProjectTree();
	
	void init(double D_d, double massScale, double widthScaleFactor,
		  const std::list<ImagesDataExtended *> &images,
		  Vector2D<double> center, double areaSize, int firstLevelSubdiv, 
		  int maxDepth, bool useWeights);
protected:
	void getDeflectionInformation(double squareSize, Vector2D<double> theta, 
			                      Vector2D<double> &alpha, 
	                                      double &alphaxx, double &alphayy, double &alphaxy);

	double m_scale;
	double m_widthScaleFactor;
};
	
} // end namespace

#endif // GRALE_PLUMMERBACKPROJECTTREE_H
