/*

  This file is a part of GRALE, a library to facilitate the simulation
  and inversion of gravitational lenses.

  Copyright (C) 2008-2012 Jori Liesenborgs

  Contact: jori.liesenborgs@gmail.com
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
*/

#ifndef GRALE_TREELENSINVERSIONGAFACTORYBASE_H

#define GRALE_TREELENSINVERSIONGAFACTORYBASE_H

#include "graleconfig.h"
#include "randomnumbergenerator.h"
#include <mogal/gafactory.h>
#include <vector>

namespace grale
{

class TreeLensInversionGAFactoryParams;
class TreeLensInversionGenomeBase;
class GridNodeBase;
class BackProjectTreeBase;
class ImagesDataExtended;

// NOTE: the virtual inheritance is again very important!
class GRALE_IMPORTEXPORT TreeLensInversionGAFactoryBase : public virtual mogal::GAFactory
{
public:
	TreeLensInversionGAFactoryBase();
	~TreeLensInversionGAFactoryBase();

	mogal::GAFactoryParams *createParamsInstance() const;
	const mogal::GAFactoryParams *getCurrentParameters() const;

	bool init(const mogal::GAFactoryParams *p);

	mogal::Genome *createNewGenome() const;

	size_t getMaximalFitnessSize() const							{ return sizeof(float)*(1+getNumberOfFitnessComponents()); } // one for scale factor, some for fitness
	size_t getMaximalGenomeSize() const							{ return m_maxGenomeSize; }

	bool writeGenome(serut::SerializationInterface &si, const mogal::Genome *g) const;
	bool readGenome(serut::SerializationInterface &si, mogal::Genome **g) const;
	bool writeGenomeFitness(serut::SerializationInterface &si, const mogal::Genome *g) const;
	bool readGenomeFitness(serut::SerializationInterface &si, mogal::Genome *g) const;
	bool writeCommonGenerationInfo(serut::SerializationInterface &si) const;
	bool readCommonGenerationInfo(serut::SerializationInterface &si);

	const RandomNumberGenerator *getRandomNumberGenerator() const				{ return &m_rndGen; }

	int getMaximumNumberOfBasisFunctions() const						{ return m_maxBasisFunctions; }
	int getMaximumDepth() const								{ return m_maxDepth; }
	bool useWeights() const									{ return m_useWeights; }

	virtual float getChanceMultiplier() = 0;
	virtual bool useAbsoluteMutation() = 0;
	virtual float getMutationAmplitude() = 0;
protected:
	// if 'pRootNode' is null, random masses should be used
	virtual TreeLensInversionGenomeBase *allocateNewGenome(TreeLensInversionGAFactoryBase *pFactory,
			                           BackProjectTreeBase *pBackProjectTree,
						   GridNodeBase *pRootNode,
						   int firstLevelSubdiv) const = 0;
	
	int getMaximumNumberOfGenerations() const						{ return m_maxGenerations; }
	virtual bool subInit(std::list<ImagesDataExtended *> &images)				{ return true; }
	
#ifdef SHOWEVOLUTION
	void onSortedPopulation(const std::vector<mogal::GenomeWrapper> &population);
#endif // SHOWEVOLUTION
private:
	bool writeNode(serut::SerializationInterface &si, const GridNodeBase *pNode) const;
	bool readNode(serut::SerializationInterface &si, GridNodeBase **pNode, int level) const;
	
	BackProjectTreeBase *m_pBackProjectTree;
	RandomNumberGenerator m_rndGen;
	TreeLensInversionGAFactoryParams *m_pCurrentParams;
	int m_maxGenerations;
	int m_maxDepth;
	int m_maxBasisFunctions;
	int m_maxGenomeSize;
	int m_firstLevelSubdiv;
	bool m_useWeights;
};
	
} // end namespace

#endif // GRALE_TREELENSINVERSIONGAFACTORYBASE_H

