/*

  This file is a part of GRALE, a library to facilitate the simulation
  and inversion of gravitational lenses.

  Copyright (C) 2008-2012 Jori Liesenborgs

  Contact: jori.liesenborgs@gmail.com
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
*/

#include "graleconfig.h"
#include "powerlawdistribution.h"
#include "realtype.h"

#include "debugnew.h"

namespace grale
{

PowerLawDistribution::PowerLawDistribution(double min, double max, double alpha, const mogal::RandomNumberGenerator *pRndGen) : ProbabilityDistribution(pRndGen)
{
	m_oneOverAlphaPlusOne = 1.0/(alpha + 1.0);
	m_minValuePower = POW(min, (alpha + 1.0));
	m_maxMinusMinPower = POW(max, (alpha + 1.0)) - m_minValuePower;
}

PowerLawDistribution::~PowerLawDistribution()
{
}

double PowerLawDistribution::pickNumber() const
{
	double y = pickRandomNumber();
	return POW(y*m_maxMinusMinPower + m_minValuePower, m_oneOverAlphaPlusOne);
}

} // end namespace

