/*

  This file is a part of GRALE, a library to facilitate the simulation
  and inversion of gravitational lenses.

  Copyright (C) 2008-2012 Jori Liesenborgs

  Contact: jori.liesenborgs@gmail.com
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
*/

#include "graleconfig.h"
#include "circularsource.h"

#include "debugnew.h"

namespace grale
{

CircularSource::CircularSource(Vector2D<double> angularpos, double angularradius, double brightnessScale) : SourceImage(SourceImage::Circle, angularpos, 0, brightnessScale)
{
	radius = angularradius;
	radius2 = angularradius*angularradius;
	fade = false;
}

CircularSource::CircularSource() : SourceImage(SourceImage::Circle, Vector2D<double>(0,0), 0, 1.0)
{
	radius = 0;
	radius2 = 0;
	fade = false;
}

CircularSource::~CircularSource()
{
}

SourceImage *CircularSource::createCopy() const
{
	CircularSource *src = new CircularSource(getAngularPosition(), radius, getBrightnessScale());
	src->fade = fade;
	return src;
}

bool CircularSource::writeInternal(serut::SerializationInterface &si) const
{
	if (!si.writeDouble(radius))
	{
		setErrorString(si.getErrorString());
		return false;
	}

	int32_t fadeval = 0;
	if (fade)
		fadeval = 1;

	if (!si.writeInt32(fadeval))
	{
		setErrorString(si.getErrorString());
		return false;
	}
	return true;
}

bool CircularSource::readInternal(serut::SerializationInterface &si)
{
	double r;
	int32_t fadeval;

	if (!si.readDouble(&r))
	{
		setErrorString(si.getErrorString());
		return false;
	}
	if (!si.readInt32(&fadeval))
	{
		setErrorString(si.getErrorString());
		return false;
	}
	if (fadeval == 0)
		fade = false;
	else
		fade = true;
	radius = r;
	radius2 = r*r;
	return true;
}

} // end namespace

